#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;

namespace HeuristicLab.Optimization.Networks {
  [Item("OutputInputPort", "A synchronous output/input port of an optimization network node.")]
  [StorableClass]
  public class OutputInputPort<TOut, TIn> : ValuePort<TOut>, IOutputInputPort<TOut, TIn>
    where TOut : class, IItem
    where TIn : class, IItem {

    [StorableConstructor]
    protected OutputInputPort(bool deserializing) : base(deserializing) { }
    protected OutputInputPort(OutputInputPort<TOut, TIn> original, Cloner cloner) : base(original, cloner) { }
    public OutputInputPort() : base("OutputInputPort") { }
    public OutputInputPort(string name) : base(name) { }
    public OutputInputPort(string name, string description) : base(name, description) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new OutputInputPort<TOut, TIn>(this, cloner);
    }

    public TIn SendValue(TOut value) {
      Value = value;
      return OnValueSent(value);
    }
    public IItem SendValue(IItem value) {
      TOut val = value as TOut;
      if ((value != null) && (val == null))
        throw new InvalidOperationException(
          string.Format("Type mismatch. Value is not a \"{0}\".",
                        typeof(TOut).GetPrettyName())
        );
      return SendValue(val);
    }

    public event EventHandler<OutputInputPortEventArgs<TOut, TIn>> ValueSent;
    protected virtual TIn OnValueSent(TOut value) {
      var args = new OutputInputPortEventArgs<TOut, TIn>(value);
      var handler = ValueSent;
      if (handler != null) handler(this, args);
      return args.Result;
    }
  }
}
